# System imports
import os
import subprocess
import json

# Local imports
from cache_managers import JsonCache


def _check_cache_file(cache, cache_path):
    # Check file exists
    assert os.path.isfile(cache_path), "Cache file not created"
    print u"\u2713 Cache file exists: {}".format(cache_path)

    # Check the data
    with open(cache_path) as cache_file:
        assert json.load(cache_file) == cache.get_cache(), "Bad data"
        print u"\u2713 Cache file data is correct"


def test_cache_managers():
    print (
        "\n===\n"
        "Test JsonCache"
        "\n===\n"
    )

    cache_path = subprocess.check_output(
        'mktemp -u /tmp/cache-XXXX.json'.split()
    ).strip()

    cache = JsonCache(cache_path=cache_path)

    fake_data = {
        'sentence': 'hello world',
        'number': 12,
        'array': [1, 2, 3, "fish"]
    }

    # Insert items using "set"
    for key, value in fake_data.iteritems():
        cache.set(key, value)

    # Check data against inserted data
    assert fake_data == cache.get_cache(), "Data not set correctly"
    print u"\u2713 Data correctly inserted"

    # Verify cache file
    _check_cache_file(cache, cache_path)

    # Check retrieving each key with "get"
    for key, value in fake_data.iteritems():
        assert cache.get(key) == value

    print u"\u2713 Successfully retrieved items with 'get'"

    # Check wiping the cache
    cache.wipe()

    assert not os.path.isfile(cache_path), "Cache file shouldn't exist!"
    assert not cache.get(fake_data.keys()[0]), "Cache still returning data"
    print u"\u2713 Cache successfully wiped"

    # Recreate cache with "put_cache"
    cache.put_cache(fake_data)

    # Check data against inserted data
    assert fake_data == cache.get_cache()
    print u"\u2713 Data correctly inserted"

    # Check file integrity
    _check_cache_file(cache, cache_path)

    # Clean up
    cache.wipe()
    assert not os.path.isfile(cache_path), "Cache file shouldn't exist!"
    print u"\u2713 Deleted {}".format(cache_path)

if __name__ == "__main__":
    test_cache_managers()
